// SPDX-License-Identifier: GPL-3.0
pragma solidity ^0.8.23;

import "@openzeppelin/contracts/access/AccessControl.sol";

contract DappIndexer is AccessControl {
    // Moderator adds/updates and removes DAPPs
    bytes32 public constant MODERATOR_ROLE = keccak256("MODERATOR_ROLE");

    struct PackedCID {
        bytes32 CID1;
        bytes32 CID2;
    }

    mapping (bytes32 => PackedCID) private dapps;

    constructor(
        address defaultAdmin,
        address moderator
    ){
        _grantRole(DEFAULT_ADMIN_ROLE, defaultAdmin);
        _grantRole(MODERATOR_ROLE, moderator);
    }

    function addDapp(bytes32 dapp_name, PackedCID calldata cid)
    external onlyRole(MODERATOR_ROLE) {
        dapps[dapp_name] = PackedCID(cid.CID1, cid.CID2); 
    }

    function removeDapp(bytes32 dapp_name)  external onlyRole(MODERATOR_ROLE) {
        delete dapps[dapp_name];
    }

    function getDapp(bytes32 dapp_name) external view returns (PackedCID memory){
        return dapps[dapp_name]; 
    }
}
